'use strict';

Object.defineProperty(exports, '__esModule', {
  value: true
});
exports['default'] = createEnhanceReducer;

var _utils = require('./utils');

/**
 * AppStateOrEffect mapper - extracts just the value.
 *
 * @param {Object} The generator iteration object
 * @returns {any} Might be either Function (for Effect) or any for Application State
 */
var mapValue = function mapValue(iteration) {
  return iteration.value;
};

/**
 * Creates a reducer enhancer which iterates over generator-like reducer reduction and
 * dispatches all the side effects while returning application state.
 *
 * @param {Function} A dispatch function which is deffered to after call stack completion.
 * @returns {Function} A reducer enhancer which takes the old reducer and returns a new one.
 */

function createEnhanceReducer(defferedDispatch) {
  return function (rootReducer) {
    return function (state, action) {
      (0, _utils.invariant)((0, _utils.isFunction)(rootReducer), 'Provided root reducer is not a function.');

      // Calling the Root reducer should return an Iterable
      var reduction = rootReducer(state, action);

      if ((0, _utils.isIterable)(reduction)) {
        // Iterable returned by Root reducer is mapped into array of values.
        // Last value in the array is reduced application state all the other values
        // are just side effects.
        var effects = (0, _utils.mapIterable)(reduction, mapValue);
        var newState = effects.pop();

        // Dispatch each effect later.
        effects.forEach(defferedDispatch);

        (0, _utils.invariant)(!(0, _utils.isUndefined)(newState), 'Root reducer does not return new application state. Undefined is returned');

        return newState;
      } else {
        console.warn('createEffectCapableStore enhancer from redux-side-effects package is used, ' + 'yet the provided root reducer is not a generator function');

        return reduction;
      }
    };
  };
}

module.exports = exports['default'];