/**
 * Simple invariant check.
 *
 * @param {Boolean} A condition to be met
 *
 * @param {String} An exception message to be thrown
 * @returns {void}
 */
'use strict';

var _Object$keys = require('babel-runtime/core-js/object/keys')['default'];

var _regeneratorRuntime = require('babel-runtime/regenerator')['default'];

Object.defineProperty(exports, '__esModule', {
  value: true
});
var invariant = function invariant(condition, message) {
  if (!condition) {
    throw new Error('Invariant violation: ' + message);
  }
};

exports.invariant = invariant;
/**
 * Checks whether provided argument is a function.
 *
 * @param {any} Anything
 *
 * @returns {Boolean} Result of function check
 */
var isFunction = function isFunction(any) {
  return typeof any === 'function';
};

exports.isFunction = isFunction;
/**
 * Checks whether provided argument is undefined.
 *
 * @param {any} Anything
 *
 * @returns {Boolean} Result of undefined check
 */
var isUndefined = function isUndefined(any) {
  return typeof any === 'undefined';
};

exports.isUndefined = isUndefined;
/**
 * Checks whether provided argument is iterable. The value must be defined and
 * must contain function named next.
 *
 * @param {any} Anything
 *
 * @returns {Boolean} Result of iterable check
 */
var isIterable = function isIterable(value) {
  return !isUndefined(value) && isFunction(value.next);
};

exports.isIterable = isIterable;
/**
 * Checks whether provided argument is generator.
 * The implementation is quite fragile. The current state
 * however does not allow reliable implementation of `isGenerator`
 * function.
 *
 * @param {any} Anything
 *
 * @returns {Boolean} Result of generator check
 */
var isGenerator = function isGenerator(fn) {
  if (isFunction(fn)) {
    var result = fn();
    return !!result && isFunction(result._invoke);
  } else {
    return false;
  }
};

exports.isGenerator = isGenerator;
/**
 * Map implementation which takes iterable as an argument.
 *
 * @param {Iterable}
 *
 * @param {Function}
 * @returns {Array} Array mapped by mapper function
 */
var mapIterable = function mapIterable(iterable, mapper) {
  invariant(isIterable(iterable), 'First argument passed to mapIterable must be iterable');

  invariant(isFunction(mapper), 'Second argument passed to mapIterable must be a function');

  // Clojure like recur loop
  // It's not ideal to use for..of as it does not
  // return the last value in iteration loop
  var recur = function recur(_x) {
    var _again = true;

    _function: while (_again) {
      var acc = _x;
      _again = false;

      var next = iterable.next();
      acc.push(mapper(next.value, next.done));

      // ES6 tail call
      if (next.done) {
        return acc;
      } else {
        _x = acc;
        _again = true;
        next = undefined;
        continue _function;
      }
    }
  };

  return recur([]);
};

exports.mapIterable = mapIterable;
/**
 * Returns first element in a non-empty array;
 *
 * @param {Array}
 * @returns {any} First element in the provided Array
 */
var first = function first(arr) {
  invariant(Array.isArray(arr), 'Provided argument is not array');

  invariant(arr.length > 0, 'Provided array is empty');

  return arr[0];
};

exports.first = first;
/**
 * Standard map implementation which works with type of Object
 *
 * @param {Object} Object to be mapped
 *
 * @param {Function} Mapper function
 * @returns {Object} Mapped object
 */
var mapObject = function mapObject(object, fn) {
  return _Object$keys(object).reduce(function (memo, key) {
    memo[key] = fn(object[key], key);

    return memo;
  }, {});
};

exports.mapObject = mapObject;
/**
 * Standard map implementation which works with type of Object
 * and mapper in form of generator
 *
 * @param {Object} Object to be mapped
 *
 * @param {Function} Mapper function
 * @returns {Object} Mapped object
 */
var generatorMapObject = _regeneratorRuntime.mark(function generatorMapObject(object, generatorFn) {
  var marked1$0, keys, recur;
  return _regeneratorRuntime.wrap(function generatorMapObject$(context$1$0) {
    while (1) switch (context$1$0.prev = context$1$0.next) {
      case 0:
        recur = function recur(memo, index) {
          return _regeneratorRuntime.wrap(function recur$(context$2$0) {
            while (1) switch (context$2$0.prev = context$2$0.next) {
              case 0:
                return context$2$0.delegateYield(generatorFn(object[keys[index]], keys[index]), 't0', 1);

              case 1:
                memo[keys[index]] = context$2$0.t0;

                if (!keys[index + 1]) {
                  context$2$0.next = 7;
                  break;
                }

                return context$2$0.delegateYield(recur(memo, index + 1), 't2', 4);

              case 4:
                context$2$0.t1 = context$2$0.t2;
                context$2$0.next = 8;
                break;

              case 7:
                context$2$0.t1 = memo;

              case 8:
                return context$2$0.abrupt('return', context$2$0.t1);

              case 9:
              case 'end':
                return context$2$0.stop();
            }
          }, marked1$0[0], this);
        };

        marked1$0 = [recur].map(_regeneratorRuntime.mark);

        invariant(isGenerator(generatorFn), 'First argument passed to filterGeneratorYieldedValues must be generator');

        keys = _Object$keys(object);
        return context$1$0.delegateYield(recur({}, 0), 't0', 5);

      case 5:
        return context$1$0.abrupt('return', context$1$0.t0);

      case 6:
      case 'end':
        return context$1$0.stop();
    }
  }, generatorMapObject, this);
});

exports.generatorMapObject = generatorMapObject;
/**
 * Returns all the yield value within generator.
 *
 * @param {Object} Iterable created of generator
 *
 * @returns {Array} Array of yielded values
 */
var filterGeneratorYieldedValues = function filterGeneratorYieldedValues(iterable) {
  invariant(isIterable(iterable), 'First argument passed to getGeneratorReturnValue must be an iterable');

  var recur = function recur(_x2) {
    var _again2 = true;

    _function2: while (_again2) {
      var acc = _x2;
      _again2 = false;

      var next = iterable.next();
      if (next.done) {
        return acc;
      } else {
        acc.push(next.value);

        _x2 = acc;
        _again2 = true;
        next = undefined;
        continue _function2;
      }
    }
  };

  return recur([]);
};

exports.filterGeneratorYieldedValues = filterGeneratorYieldedValues;
/**
 * Returns the returned value withing generator
 *
 * @param {Object} Iterable created of generator
 *
 * @returns {any} Value returned in generator
 */
var getGeneratorReturnValue = function getGeneratorReturnValue(iterable) {
  invariant(isIterable(iterable), 'First argument passed to getGeneratorReturnValue must be generator');

  var recur = function recur() {
    var _again3 = true;

    _function3: while (_again3) {
      _again3 = false;

      var next = iterable.next();
      if (next.done) {
        return next.value;
      } else {
        _again3 = true;
        next = undefined;
        continue _function3;
      }
    }
  };

  return recur();
};
exports.getGeneratorReturnValue = getGeneratorReturnValue;